/**
 * @file
 * Custom JS for the JSON table field formatter.
 */

(function ($, Drupal, once) {
  'use strict';

  /**
   * Attach behavior for JSON Fields.
   *
   * @type {Drupal~behavior}
   */
  function loadData(that){
    let data = that.find('textarea.json-table').val();
    let tbody = that.find('table.json-table tbody');
    let countTr = tbody.find('tr').length;
    if (data != '' && countTr == 1) {
      tbody.html('');
      let dataTable = JSON.parse(data);
      if(typeof dataTable === 'object') {
        dataTable = Object.values(dataTable);
      }
      dataTable.forEach((rows,row) => {
        if(typeof rows === 'object') {
          rows = Object.values(rows);
        }
        let tr = $('<tr></tr>');
        rows.forEach((v3,col) => {
          let cell = 'cell-' + row + '-' + col;
          let td = $('<td><input class="form-control form-element form-text ' + cell + '" value="' + v3 + '"/></td>');
          td.appendTo(tr);
        });
        tr.appendTo(tbody);
      });
    }
  }
  function getHeaderArray(that){
    var header = [];
    let $table = that.closest('table'),
      header_associative = $table.data('associative');
    if(header_associative == '1'){
      // This loop is not very efficient. It crashes browser.
      $table.find('thead th').each(function (key,value) {
        header.push(value.innerHTML);
      });
    }
    return header;
  }
  function convertTable2Json(that) {
    let $textarea = that.closest('.field--widget-json-table-widget').find('textarea.json-table');
    if(!$textarea.length) {
      $textarea = that.closest('.description').parent().find('textarea.json-table');
    }
    var table = [];
    var header = getHeaderArray(that);
    that.closest('table').find('tbody tr').each(function (row, tr) {

      let rows = $(this).find('td');
      table[row] = {};
      $.each(rows, function (col, value) {
        let associative = col;
        if(header[col] !== undefined){
          associative = header[col];
        }
        table[row][associative] = $(this).find('input,select,textarea').val();
      });
    });
    console.log($textarea)

    $textarea.val(JSON.stringify(table));
  }
  function keyupInputTable(selector = null){
    if(selector === null) {
      selector = '.json-table input,.json-table select,.json-table textarea';
    }
    $(selector).on('change keyup', function (e) {
      convertTable2Json($(this))
    });
  }
  keyupInputTable();
  Drupal.behaviors.json_table = {
    attach: function (context) {
      // Initialize the Quick Edit app once per page load.
      $(once('json-table-field', '.field--type-json.field--widget-json-table-widget', context)).each(function () {
        $(once('table-add-row', '.add-row', context)).on('click', function () {
          let tbody = $(this).closest('.json-table-wrapper').find('table tbody');
          let tr = tbody.children().last().clone();
          tbody.append(tr);
          pasteFromClipBoard();
          keyupInputTable($(this).parent().find('.json-table input,.json-table select,.json-table textarea'));
          convertTable2Json(tbody);
        });
        $(once('table-add-col', '.add-col', context)).on('click', function () {
          let tbody = $(this).closest('.json-table-wrapper').find('table tbody');
          tbody.find('tr').append('<td><input class="form-control form-element form-text"/></td>');
          keyupInputTable($(this).parent().find('.json-table input,.json-table select,.json-table textarea'));
          convertTable2Json(tbody);
        });
        $(once('table-remove-row', '.remove-row', context)).on('click', function () {
          let tbody = $(this).closest('.json-table-wrapper').find('table tbody');
          tbody.find('tr:last').remove();
          convertTable2Json(tbody);
        });
        $(once('table-remove-col', '.remove-col', context)).on('click', function () {
          let tbody = $(this).closest('.json-table-wrapper').find('table tbody');
          tbody.find('tr td:last-child').remove();
          convertTable2Json(tbody);
        });
        pasteFromClipBoard();
        keyupInputTable();
        loadData($(this));
        function pasteFromClipBoard() {
          $(once('paste','.json-table input.paste')).on('paste', function (e) {
            var $this = $(this),
              header = getHeaderArray($this);
            $.each(e.originalEvent.clipboardData.items, function (i, v) {
              if (v.type === 'text/plain') {
                v.getAsString(function (text) {
                  if(text.includes('\r\n') || text.includes('\t')){
                    let tbody = $this.closest('tbody');
                    let $textarea = $this.closest('.form-type-textarea').find('textarea');
                    let table = [];
                    text = text.trim('\r\n');
                    tbody.html('');
                    text.split('\r\n').forEach((v2,row) => {
                      let rows = v2.split('\t');
                      table[row] = {};
                      let tr = $('<tr></tr>');
                      rows.forEach((v3,col) => {
                        let cell = 'cell-' + row + '-' + col;
                        let td = $('<td><input class="form-control form-element form-text ' + cell + '" value="' + v3 + '"/></td>');

                        let associative = col;
                        if(header[col] !== undefined){
                          associative = header[col];
                        }
                        table[row][associative] = v3;
                        td.appendTo(tr);
                      });
                      tr.appendTo(tbody);
                    });
                    $textarea.val(JSON.stringify(table));
                  }
                });
              }
            });
          });
        }
      });

      $(once('lock', '.lock', context)).on('click', function () {
        let state = $(this).data('lock');
        let table = $(this).parent().parent().find('table tbody');
        if (state == 'active') {
          $(this).data('lock', 'inactive');
          $(this).text('🔓');
          table.find('label').addClass('js-hide');
          table.find('input.js-hide').removeClass('js-hide');
        } else {
          $(this).data('lock', 'active');
          $(this).text('🔒');
          let label = table.find('label');
          label.removeClass('js-hide');
          label.parent().find('input').addClass('js-hide');
        }
      });
    }
  };
}(jQuery, Drupal, once));
